% Binary Distillation Column model found in
%
% Diehl, M., "Real-Time Optimization for Large Scale Nonlinear Processes", PhD thesis, University
%   of Heidelberg, 2001.
% The model is a 204 state DAE with 82 differential equations and 122 algebraic equations.
%
% and
%
% Diehl, M., et. al. "Real-Time Optimization for Large Scale Nonlinear Processes: Nonlinear 
%   Model Predictive Control of a High Purity Distillation Column", In Groetschel, Krumke,
%   Rambau (editors): Online Optimization of Large Scale Systems: State of the Art, Springer 2001.
% The model is a 164 state DAE with 42 differential equations and 122 algebraic equations. This model
%   was used to control the pilot plant distillation column.  It is a simplification from Diehl's thesis
%   work in that constant molar holdup is assumed for each tray.
%
% For NMPC - t_control = 600 sec with 5 control intervals of 120 sec
% The iteration time constraint was 20 sec on a 1009 MHz AMD Athlon
%
%
% The model found here uses the data from the above two references but with one modification.  The liquid
%   flowrate from each tray can be solved explicitly thereby eliminating 40 algebraic equations.  The total
%   for this model is 42 differential equations and 125 algebraic equations.  The extra equation is an
%   energy balance to determine the vapor flowrate from the reboiler.
%
%
%  Component A = Methanol
%  Component B = n-Propanol
%
%  t -- time (not used)
%  x -- states
%  xdot -- state derivatives

function xdot = distill(t,x,flag)

global u x_ss

if isempty(flag),
   
% Inputs (2)
% Input 1: Heat Input to the Reboiler from an Electric Heater (J/sec)
Q_elec = u(1,1);
% Input 2: Flow Rate of the Recycled Distillate (m^3/sec)
Vdot_L1 = u(2,1);
%
% States (204):
offset = 0;
% Oridinary Differential Equations (ODEs) =============================
% x(1) - Reflux Drum Liquid Mole Fraction of Component A
% x(2) - Tray 1 - Liquid Mole Fraction of Component A
% .
% .
% .
% x(22) - Tray 21 - Liquid Mole Fraction of Component A (Feed Location)
% .
% .
% .
% x(41) - Tray 40 - Liquid Mole Fraction of Component A
% x(42) - Reboiler Liquid Mole Fraction of Component A
for i = 1:42,
   xA(i,1) = x(i+offset,1);
end
offset = offset + 42;
% ---------------------------------------------------------------------
% ---------------------------------------------------------------------
% Algebraic Equations (AEs) ===========================================
% x(43) - Tray 1 - Vapor Molar Flux (mol/sec)
% .
% .
% .
% x(63) - Tray 21 - Vapor Molar Flux (mol/sec) (Feed Location)
% .
% .
% .
% x(82) - Tray 40 - Vapor Molar Flux (mol/sec)
% x(83) - Reboiler - Vapor Molar Flux (mol/sec)
for i = 1:41,
   V(i+1,1) = x(i+offset,1);
end
offset = offset + 41;
% ---------------------------------------------------------------------
% Algebraic Equations (AEs) ===========================================
% x(84) - Condenser Temperature (K)
% x(85) - Tray 1 - Temperature (K)
% .
% .
% .
% x(105) - Tray 16 - Temperature (K)
% .
% .
% .
% x(124) - Tray 30 - Temperature (K)
% x(125) - Reboiler Temperature (K)
for i = 1:42,
   Temp(i,1) = x(i+offset,1);
end
offset = offset + 42;
% ---------------------------------------------------------------------
%
% Parameters - Nominal Operating Conditions
% Feed Flowrate 
Vdot_Feed =  14.0 / 3600 / 1000; % (m^3/sec)
% Mole Fraction of Feed
xA_Feed = 0.32;
% Feed Temperature
Temp_Feed = 71.0 + 273.15; % (K)
% Condenser Temperature (Water Cooled Heat Exchanger)
% If I specify this then I need to take away the Temperature equation for the condenser
%Temp(1,1) = 47.2 + 273.15; % (K)
% Top Pressure 
P_top = 0.97 * 1.0e5; % (Pa)
% Condenser Holdup 
V_Condenser = 0.17 / 1000; % (m^3)
% Reboiler Holdup 
V_Reboiler = 8.5 / 1000; % (m^3)
% Heat Loss from the Reboiler 
Q_loss = 0.51e3; % (J/sec)
% Pressure Drop in the Rectification (Above Feed) and Stripping (Below Feed) Sections
dP_rect = 1.9e3; % (Pa)
dP_strip = 2.5e3; % (Pa)
% Tray Efficiencies
alpha_rect = 0.35;
alpha_strip = 0.62;
%
% Pressures
Press(1,1) = P_top;
for i = 2:21,
   Press(i,1) = Press(i-1,1) + dP_rect;
end
for i = 22:42,
   Press(i,1) = Press(i-1,1) + dP_strip;
end
%
% Parameters - Physical Properties
%
% Molar Volume (m^3/mol) of Component A and B
% Units (mol/m^3)
a1_A = 2.288e3;
a1_B = 1.235e3;
% Units (dimensionless)
b1_A = 0.2685;
b1_B = 0.27136;
% Units (K)
c1_A = 512.4;
c1_B = 536.4;
% Units (dimensionless)
d1_A = 0.2453;
d1_B = 0.2400;
%
% Pure Component Molar Volumes (m^3/mol)
for i = 1:42,
   V_mol_A(i,1) = (1 / a1_A) * (b1_A ^ (1 + (1 - (Temp(i,1) / c1_A)) ^ d1_A));
   V_mol_B(i,1) = (1 / a1_B) * (b1_B ^ (1 + (1 - (Temp(i,1) / c1_B)) ^ d1_B));
end
% Mixture Molar Volumes (m^3/mol)
for i = 1:42,
   V_mol(i,1) = xA(i,1) * V_mol_A(i,1) + (1 - xA(i,1)) * V_mol_B(i,1);
end
% Feed Tray Molar Volume (m^3/mol)
V_mol_A_Feed = (1 / a1_A) * (b1_A ^ (1 + (1 - (Temp_Feed / c1_A)) ^ d1_A));
V_mol_B_Feed = (1 / a1_B) * (b1_B ^ (1 + (1 - (Temp_Feed / c1_B)) ^ d1_B));
V_mol_Feed = xA_Feed * V_mol_A(i,1) + (1 - xA_Feed) * V_mol_B(i,1);
%
%
% Feed Tray Molar Flowrate (mol/sec)
Feed =  Vdot_Feed / V_mol_Feed;
%
% Feed Flow rates (mol/sec)
for i = 1:42,
   F(i,1) = 0;
end
F(22,1) = Feed;
%
%
% Condenser Recycle Molar Flowrate (mol/sec)
L(1,1) =  Vdot_L1 / V_mol(1,1);
%
% Solve for the Liquid Flow rates (mol/sec) explicitly
for i = 2:41,
   L(i,1) = L(i-1,1) + V(i+1,1) - V(i,1) + F(i,1);
end
%
% Distillate Molar Flowrate Flowrate (mol/sec)
% Assume Constant Molar Holdup for the Condenser
Dist = V(2,1) - L(1,1);
%
% Determine the Bottoms Molar Flow Rate
% Here I could use a mole balance around the entire column instead
% Assume Constant Molar Holdup for the Reboiler
Bott = L(41,1) - V(42,1);
%
% Condenser Liquid Molar Holdup (mol) - Assume this is constant
%nL(1,1) = 4.0; % Number calculated from holdup of a 0.17 L condenser at xA = 0.99 and T = 333 K 
% Reboiler Liquid Molar Holdup (mol) - Assume this is constant
%nL(42,1) = 104; % Number calculated from holdup of a 8.5 L reboiler at xA = 0.01 and T = 371 K
% Constant Molar Holdup For Each Tray
% Assumptions:
% 1. Column is a cylinder with D = 0.1m and H = 7m
% 2. Half the column is liquid
% 3. xA = 0.99 at the top and 0.01 at the bottom with a range of values in between
% 4. T = 333 K at the top and 371 K at the bottom with a range of values in between
% 5. The liquid is evenly distributed amoung 40 trays
%nL = [
%4.0
%16.0033585
%15.77230727
%15.69300879
%15.4783882
%15.2996413
%15.01527479
%14.9784638
%14.82509056
%14.56796115
%14.32132105
%13.87727793
%13.56011556
%13.16299724
%12.70299163
%12.22674212
%11.86043105
%11.4800095
%10.87615919
%10.45358225
%10.27179296
%10.10648189
%9.945445748
%9.795621795
%9.68921543
%9.534080941
%9.384810637
%9.24014418
%9.143659945
%9.048357367
%8.954355831
%8.904149678
%8.812013179
%8.762461843
%8.672328372
%8.62365872
%8.575218254
%8.526964458
%8.478888461
%8.430954452
%8.383439376
%104
%];

% From Moritz Diehl (not in the articles)
for i = 1:41,
   nL(i,1) = 2.1221253477356540;
end
nL(42,1) = 113.000;
%
% Pure Component A Saturation Pressure - Antione Equation
% Units (dimensionless)
a2_A = 23.48;
a2_B = 22.437;
% Units (K)
b2_A = 3626.6;
b2_B = 3166.4;
% Units (K)
c2_A = -34.29;
c2_B = -80.15;
%
% Pure Component Saturation Pressures (Pa)
for i = 1:42,
   Psat_A(i,1) = exp(a2_A - b2_A/(Temp(i,1) + c2_A));
   Psat_B(i,1) = exp(a2_B - b2_B/(Temp(i,1) + c2_B));
end
%
% Enthalpy Coefficients
% Units (1/K)
h1_A = 18.31;
h1_B = 31.92;
% Units (1/K^2)
h2_A = 1.713e-2;
h2_B = 4.49e-2;
% Units (1/K^3)
h3_A = 6.399e-5;
h3_B = 9.663e-5;
% Units (K)
Tc_A = 512.6;
Tc_B = 536.7;
% Units (Pa)
Pc_A = 8.096e6;
Pc_B = 5.166e6;
% Units (dimensionless)
Omega_A = 0.557;
Omega_B = 0.612;
% Units (K)
To = 273.15;
% Units (J/mol)
c1 = 4.186;
% Universal Gas Law Constant (J/mol-K)
Univ_R = 8.31451;
% Units (dimensionless)
c2 = [6.09648 1.28862 1.016 15.6875 13.4721 2.615];
%
% Pure Component Liquid Enthalpies (J/mol)
for i = 1:42,
   hL_A(i,1) = c1 * (h1_A * (Temp(i,1) - To) + h2_A * (Temp(i,1) - To)^2 + ...
      h3_A * (Temp(i,1) - To)^3);
   hL_B(i,1) = c1 * (h1_B * (Temp(i,1) - To) + h2_B * (Temp(i,1) - To)^2 + ...
      h3_B * (Temp(i,1) - To)^3);
end
% Mixture Liquid Enthalpies (J/mol)
for i = 1:42,
   hL(i,1) = xA(i,1) * hL_A(i,1) + (1 - xA(i,1)) * hL_B(i,1);
end
% Feed Enthalpy
hL_A_Feed = c1 * (h1_A * (Temp_Feed - To) + h2_A * (Temp_Feed - To)^2 + ...
   h3_A * (Temp_Feed - To)^3);
hL_B_Feed = c1 * (h1_B * (Temp_Feed - To) + h2_B * (Temp_Feed - To)^2 + ...
   h3_B * (Temp_Feed - To)^3);
hL_Feed = xA_Feed * hL_A_Feed + (1 - xA_Feed) * hL_B_Feed;
%
% Pure Component Vapor Enthalpies (J/mol)
for i = 1:42,
   hV_A(i,1) = hL_A(i,1) + Univ_R * Tc_A * sqrt(abs(1 - (Press(i,1)/Pc_A) / (Temp(i,1)/Tc_A)^3)) * ...
      (c2(1) - c2(2)*(Temp(i,1)/Tc_A) + c2(3)*(Temp(i,1)/Tc_A)^7 + ...
      Omega_A * (c2(4) - c2(5)*(Temp(i,1)/Tc_A) + c2(6)*(Temp(i,1)/Tc_A)^7));
   hV_B(i,1) = hL_B(i,1) + Univ_R * Tc_B * sqrt(abs(1 - (Press(i,1)/Pc_B) / (Temp(i,1)/Tc_B)^3)) * ...
      (c2(1) - c2(2)*(Temp(i,1)/Tc_B) + c2(3)*(Temp(i,1)/Tc_B)^7 + ...
      Omega_A * (c2(4) - c2(5)*(Temp(i,1)/Tc_B) + c2(6)*(Temp(i,1)/Tc_B)^7));
end
% Mixture Vapor Enthalpies (J/mol)
for i = 1:42,
   hV(i,1) = xA(i,1) * hV_A(i,1) + (1 - xA(i,1)) * hV_B(i,1);
end
%
%
% Equations
% 
% Raoult's Law for Phase Equilibrium
for i = 2:41,
   yA_equil(i,1) = xA(i,1) * Psat_A(i,1) / Press(i,1);
end
%
% Murphee Efficiency for Vapor-Liquid Mixture Deviation from Equilibrium
% The vapor leaving the reboiler is at equilibrium with the liquid in the reboiler
yA(42,1) = xA(42,1) * Psat_A(i,1) / Press(i,1);
% The vapor passing through the liquid gets (alpha * 100%) of the way to equilibrium
for i = 41:-1:22,
   yA(i,1) = alpha_strip * (yA_equil(i,1) - yA(i+1,1)) + yA(i+1,1);
end
for i = 21:-1:2,
   yA(i,1) = alpha_rect * (yA_equil(i,1) - yA(i+1,1)) + yA(i+1,1);
end
%
%
% Mole Balance for A at Each State
% Condenser
xA_dot(1,1) = 1/nL(1,1) * (yA(2,1) * V(2,1) - xA(1,1) * L(1,1) - xA(1,1) * Dist);
% Trays
for i = 2:41,
   xA_dot(i,1) = 1/nL(i,1) * (yA(i+1,1) * V(i+1,1) + xA(i-1,1) * L(i-1,1) - yA(i,1) * V(i,1) - ...
      xA(i,1) * L(i,1) + xA_Feed * F(i,1));
end
% Reboiler
xA_dot(42,1) = 1/nL(42,1) * (xA(41,1)*L(41,1) - xA(42,1)*Bott - yA(42,1)*V(42,1));
%
%
% Enthalpy Balance at Each Stage - Algebraic Equations to Determine the Vapor Flow Rates
% Compute h_dot
for i = 2:42,
   % Partial of Liquid Enthalpy w.r.t. xA
   dh_dxA(i,1) = hL_A(i,1) - hL_B(i,1);
   dhL_A_dTemp(i,1) = c1 * (h1_A + 2 * h2_A * (Temp(i,1) - To) + ...
      3 * h3_A * (Temp(i,1) - To)^2);
   dhL_B_dTemp(i,1) = c1 * (h1_B + 2 * h2_B * (Temp(i,1) - To) + ...
      3 * h3_B * (Temp(i,1) - To)^2);
   % Partial of Liquid Enthalpy w.r.t. Temp
   dh_dTemp(i,1) = xA(i,1) * dhL_A_dTemp(i,1) + (1 - xA(i,1)) * dhL_B_dTemp(i,1);
   % Temperature derivatives from implicit function theorem of the sum of partial pressures equation
   dPsat_A_dTemp(i,1) = Psat_A(i,1) * b2_A / (Temp(i,1) + c2_A)^2;
   dPsat_B_dTemp(i,1) = Psat_B(i,1) * b2_B / (Temp(i,1) + c2_B)^2;
   Temp_dot(i,1) = - (Psat_A(i,1) - Psat_B(i,1)) * xA_dot(i,1) / ...
      (dPsat_A_dTemp(i,1) * xA(i,1) + dPsat_B_dTemp(i,1) * (1 - xA(i,1)));
   % Enthalpy Total Differential
   h_dot(i,1) = dh_dTemp(i,1) * Temp_dot(i,1) + dh_dxA(i,1) * xA_dot(i,1);
end
% Set up Algebraic Equations
% Trays
for i = 2:41,
   V_eqn(i,1) = (1/nL(i,1) * (hV(i+1,1) * V(i+1,1) + hL(i-1,1) * L(i-1,1) - hV(i,1) * V(i,1) - ...
      hL(i,1) * L(i,1) + hL_Feed * F(i,1)) - h_dot(i,1));
end
% Reboiler
V_eqn(42,1) = 1/nL(42,1) * (hL(41,1)*L(41,1) - hL(42,1)*Bott - hV(42,1)*V(42,1) + Q_elec - Q_loss) - h_dot(42,1);
%
%
%
% Algebraic equations to determine the temperature at each stage
% Press(i,1) is added as a divisor for scaling purposes
for i = 1:42,
   Temp_eqn(i,1) = (Press(i,1) - Psat_A(i,1) * xA(i,1) - Psat_B(i,1) * (1 - xA(i,1))) / Press(i,1);
end
%
% xdot must be a column vector
%
xdot = [xA_dot(1:42,1); V_eqn(2:42,1); Temp_eqn(1:42,1)];  
%
else  % Flag is not empty
   %   
   %Truncate flag for binary data operations (if statement needs strings the same length)
   if flag(1:4) == 'mass'
   	% Return M
   	M = zeros(125,125);
   	for i = 1:42,
      	M(i,i) = 1;
   	end
      xdot = M;
   end
   if flag(1:4) == 'jpat'
      start_time = cputime;
      % Return dF_dx_ss
      size_x_ss = size(x_ss,1);
      F_t_x_ss = distill(t,x_ss,'');
      jac_tol = 1e-10 * ones(size_x_ss,1);
      dF_dx_ss = numjac('distill',t,x_ss,F_t_x_ss,jac_tol,[]);
      % Determine ones and zeros
      counter = 0;
      for i = 1:size_x_ss,
         for j = 1:size_x_ss,
            if abs(dF_dx_ss(i,j)) > 1e-8,
               dF_dx_ss(i,j) = 1;
               counter = counter + 1;   
            else
               dF_dx_ss(i,j) = 0;
            end
         end
      end
      xdot = dF_dx_ss;
      fraction_nonzero = counter / size_x_ss^2;
      fraction_nonzero
      'numjac time'
      (cputime - start_time)
   end
end
