% Step test for Model 18 - Binary Distillation Column
% Created by John Hedengren
clear

global u x_ss

% Steady State Initial Condition for the Control
% Inputs (2)
% Input 1: Heat Input to the Reboiler from an Electric Heater (J/sec)
Q_elec_ss = 2.5e3;
% Input 2: Flow Rate of the Recycled Distillate (m^3/sec)
Vdot_L1_ss = 4.3 /1000/3600;

u_ss = [Q_elec_ss; Vdot_L1_ss];

% Steady State Initial Conditions for the States
load 'initial_conditions' x_ss;

% Open Loop Step Change
u = [2.45e3; 4.3 /1000/3600];

% Final Time (sec)
tf = 100 * 60;

% Set options to tell ode15s that there is a DAE coming down the pipe
opts = odeset('Mass','M','MassSingular','yes','JPattern','off');

start_time = cputime;
[t,x] = ode15s('distill',[0 tf],x_ss,opts);
(cputime - start_time)

% Distillate Composition
x1 = x(:,1)';

% Temperature of Tray 13 (Tray 28 in Reference Article)
T13 = x(:,97)';

% Temperature of Tray 27 (Tray 14 in Reference Article)
T27 = x(:,111)';

close all;
% Plot the results
figure(1);
plot(t,x1);
ylabel('Composition (x_A)');
xlabel('Time (sec)');

figure(2);
hold on;
plot(t,T13,t,T27);
ylabel('Temperature (K)');
xlabel('Time (sec)');
legend('Tray 13','Tray 27');

% Steady-State Results
size_x =size(x,1);
xA_ss = x(size_x,1:42)';
V_ss = x(size_x,43:83)';
for i = 1:41,
   V_ss(i+1,1) = V_ss(i,1);
end
V_ss(1,1) = 0;
T_ss = x(size_x,84:125)';
% Calculate Liquid Molar Flowrates
% Molar Volume (m^3/mol) of Component A and B
% Units (mol/m^3)
a1_A = 2.288e3;
a1_B = 1.235e3;
% Units (dimensionless)
b1_A = 0.2685;
b1_B = 0.27136;
% Units (K)
c1_A = 512.4;
c1_B = 536.4;
% Units (dimensionless)
d1_A = 0.2453;
d1_B = 0.2400;
%
% Pure Component Molar Volumes (m^3/mol)
for i = 1:42,
   V_mol_A(i,1) = (1 / a1_A) * (b1_A ^ (1 + (1 - (T_ss(i,1) / c1_A)) ^ d1_A));
   V_mol_B(i,1) = (1 / a1_B) * (b1_B ^ (1 + (1 - (T_ss(i,1) / c1_B)) ^ d1_B));
end
% Mixture Molar Volumes (m^3/mol)
for i = 1:42,
   V_mol(i,1) = xA_ss(i,1) * V_mol_A(i,1) + (1 - xA_ss(i,1)) * V_mol_B(i,1);
end
Temp_Feed = 71.0 + 273.15; % (K)
xA_Feed = 0.32;
% Feed Tray Molar Volume (m^3/mol)
V_mol_A_Feed = (1 / a1_A) * (b1_A ^ (1 + (1 - (Temp_Feed / c1_A)) ^ d1_A));
V_mol_B_Feed = (1 / a1_B) * (b1_B ^ (1 + (1 - (Temp_Feed / c1_B)) ^ d1_B));
V_mol_Feed = xA_Feed * V_mol_A(i,1) + (1 - xA_Feed) * V_mol_B(i,1);
%
% Feed Stream
Vdot_Feed =  14.0 / 3600 / 1000; % (m^3/sec)
% Feed Tray Molar Flowrate (mol/sec)
Feed =  Vdot_Feed / V_mol_Feed;
%
% Feed Flow rates (mol/sec)
for i = 1:42,
   F_ss(i,1) = 0;
end
F_ss(22,1) = Feed;
%
%
L_ss(1,1) = u(2,1)/V_mol(1,1);
for i = 2:41,
   L_ss(i,1) = L_ss(i-1,1) + V_ss(i+1,1) - V_ss(i,1) + F_ss(i,1);
end
%
% Set up vectors for plotting
for i = 1:42,
   y_axis42(i,1) = i;
end

figure(3);
plot(xA_ss,y_axis42);
ylabel('Stage');
xlabel('Composition (x_A)');
title('Steady-State Profile');

figure(4);
plot(V_ss(2:42,1),y_axis42(2:42,1),L_ss(1:41,1),y_axis42(1:41,1));
ylabel('Stage');
xlabel('Molar Flowrate (mol/sec)');
legend('Vapor','Liquid');
title('Steady-State Profile');


figure(5);
plot(T_ss,y_axis42);
ylabel('Stage');
xlabel('Temperature (K)');
title('Steady-State Profile');


